/*
    This file is part of the ChipWhisperer Example Targets
    Copyright (C) 2012-2017 NewAE Technology Inc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "hal.h"
#include "simpleserial.h"
#include <stdint.h>
#include <stdlib.h>

#include "api.h"
#include "crypto_aead.h"

uint8_t key[16];
uint8_t pt[16];
uint8_t nonce[16];
uint8_t ad[16];
uint64_t increment=1;

uint8_t get_key(uint8_t* k, uint8_t len)
{
	for(int i=0;i<16;i++){
        key[i]=k[i];
    }
	return 0x00;
}

uint8_t get_nonce(uint8_t* n, uint8_t len)
{
	for(int i=0;i<16;i++){
        nonce[i]=n[i];
    }
	return 0x00;
}

uint8_t get_ad(uint8_t* a, uint8_t len)
{
	for(int i=0;i<16;i++){
        ad[i]=a[i];
    }
	return 0x00;
}

uint8_t get_pt(uint8_t* pt, uint8_t len)
{
    uint8_t ct[32];
    unsigned long long clen;

//	trigger_high();

	crypto_aead_encrypt(ct, &clen, pt, 16, ad, 16, (void*) 0, nonce, key); /* encrypting the data block */
	trigger_low();

	simpleserial_put('r', 32, ct);
	return 0x00;
}

uint8_t get_pt_nonce(uint8_t* pt, uint8_t len){
    uint8_t out[48];
    unsigned long long clen;
    uint64_t *nonce_counter=(uint64_t*) nonce;
    for(int i=32;i<48;i++) out[i]=nonce[i-32];
	crypto_aead_encrypt(out, &clen, pt, 16, ad, 16, (void*) 0, nonce, key); /* encrypting the data block */
    nonce_counter[1]+=increment;
	trigger_low();
    
	simpleserial_put('r', 48, out);
	return 0x00;

}

uint8_t get_ct(uint8_t* ct, uint8_t len)
{
    uint8_t pt[17];
    unsigned long long plen;
    int res;

//	trigger_high();
	res=crypto_aead_decrypt(pt, &plen, (void*) 0, ct, 32, ad, 16, nonce, key); /* encrypting the data block */
//	trigger_low();
    
    // nonzero==tag authenticated successfully
    res|=res>>16;
    pt[16]=(res|(res>>8))&0xff;

	simpleserial_put('r', 17, pt);
	return 0x00;
}

uint8_t reset(uint8_t* x, uint8_t len)
{
    for(int i=0;i<16;i++){
        key[i]=0;
        ad[i]=0;
        pt[i]=0;
        nonce[i]=0;
    }
	return 0x00;
}


uint8_t rd_addr(uint8_t *x, uint8_t len){
    uint32_t addr,sz;
    addr=x[3]<<24;
    addr|=x[2]<<16;
    addr|=x[1]<<8;
    addr|=x[0];
    sz=x[7]<<24;
    sz|=x[6]<<16;
    sz|=x[5]<<8;
    sz|=x[4];
    simpleserial_put('d',sz, (uint8_t*)addr);\
    return 0x00;
}

int main(void)
{
    platform_init();
    reset((uint8_t*)0, 0);
    init_uart();
    trigger_setup();

	simpleserial_init();
    simpleserial_addcmd('k', 16, get_key);
    simpleserial_addcmd('p', 16,  get_pt);
    simpleserial_addcmd('c', 32,  get_ct);
    simpleserial_addcmd('n', 16, get_nonce);
    simpleserial_addcmd('d', 16,  get_pt_nonce);
    simpleserial_addcmd('a', 16,  get_ad);
    simpleserial_addcmd('x',  0,   reset);
    simpleserial_addcmd('m',  8, rd_addr);
    while(1)
        simpleserial_get();
}
